/// <reference types="react" />
import * as React from 'react';
import { Alignment, IndexRange, OverscanIndexRange, ScrollEventData } from 'react-virtualized';
import { OverscanIndicesGetter } from 'react-virtualized/dist/es/Grid';
import { CellPosition, NodeGetter, RowMouseEventHandler, RowRendererParams } from './types';
import { Update } from './utils';
export interface TreeProps {
    'aria-label'?: string;
    /**
     * Removes fixed height from the "scrollingContainer" so that the total height
     * of rows can stretch the window. Intended for use with "WindowScroller" HOC.
     */
    autoHeight?: boolean;
    /** Optional CSS class name */
    className?: string;
    /** Optional CSS class for all toggle controls */
    controlClassName?: string;
    /** Optional inline styles for all toggle controls */
    controlStyle?: React.CSSProperties;
    /**
     * Used to estimate the total height of a Tree before all of its rows have actually been measured.
     * The estimated total height is adjusted as rows are rendered.
     */
    estimatedRowSize?: number;
    /**
     * Sets height of a container Tree is rendered inside of. It also determines how many actual rows
     * are rendered.
     */
    height: number;
    /** Custom HTML id */
    id?: string;
    /**
     * Generator flattens tree data to display it in a list form basing on information of nodes' openness.
     * It should yield node metadata or node id depending on boolean "refresh" parameter it receives.
     *
     * @example
     * ```typescript
     * function * nodeGetter(refresh: boolean): IterableIterator<Node | string | null> {
     *   const stack = [];
     *
     *   stack.push({
     *     nestingLevel: 0,
     *     node: root, // tree data structure
     *   });
     *
     *   while (stack.length !== 0) {
     *     const {node, nestingLevel} = stack.pop()!;
     *     const id = node.id.toString();
     *
     *     const isOpened = yield (
     *       refresh ? {
     *         childrenCount: node.children.length,
     *         height: 20, // only if you want to use dynamic heights; otherwise use "rowHeight"
     *         id,
     *         isOpenedByDefault: true,
     *         nestingLevel,
     *         nodeData: node.name,
     *       } : id
     *     );
     *
     *     if (node.children.length !== 0 && isOpened) {
     *       for (let i = node.children.length - 1; i >= 0; i--) {
     *         stack.push({
     *           nestingLevel: nestingLevel + 1,
     *           node: node.children[i],
     *         });
     *       }
     *     }
     *   }
     * }
     * ```
     */
    nodeGetter: NodeGetter;
    /**
     * Multiplier for a margin that depends on node's nesting level. E.g. if multiplier is 10 margin for node with
     * nesting level 2 will be 20px and for node with nesting level 3 - 30px.
     */
    nodeNestingMultiplier?: number;
    /** Optional renderer to be used in place of rows when tree is empty */
    noRowsRenderer?: () => React.ReactNode | null;
    /** Callback invoked when a user clicks on a node */
    onRowClick?: RowMouseEventHandler;
    /** Callback invoked when a user double-clicks on a node */
    onRowDoubleClick?: RowMouseEventHandler;
    /** Callback invoked when the mouse leaves a node */
    onRowMouseOut?: RowMouseEventHandler;
    /** Callback invoked when a user moves the mouse over a node */
    onRowMouseOver?: RowMouseEventHandler;
    /** Callback invoked when a user right-clicks on a node */
    onRowRightClick?: RowMouseEventHandler;
    /** Callback invoked with information about the slice of rows that were just rendered */
    onRowsRendered?: (info: IndexRange & OverscanIndexRange) => void;
    /**
     * Callback invoked whenever the scroll offset changes within the inner scrollable region.
     * This callback can be used to sync scrolling between lists, tables, or grids.
     */
    onScroll?: (params: ScrollEventData) => void;
    /** See Grid#overscanIndicesGetter */
    overscanIndicesGetter?: OverscanIndicesGetter;
    /**
     * Number of rows to render above/below the visible bounds of the list.
     * These rows can help for smoother scrolling on touch devices.
     */
    overscanRowCount?: number;
    /** Optional CSS class to apply to all rows */
    rowClassName?: string;
    /** Fixed row height */
    rowHeight: number;
    /** Renders data received from NodeGetter */
    rowRenderer?: (params: RowRendererParams) => React.ReactElement<any>;
    /** Optional inline styles for all rows */
    rowStyle?: React.CSSProperties;
    /** See Grid#scrollToAlignment */
    scrollToAlignment?: Alignment;
    /** Row index to ensure visible (by forcefully scrolling if necessary) */
    scrollToIndex?: number;
    /** Vertical offset */
    scrollTop?: number;
    /** Optional inline style */
    style?: React.CSSProperties;
    /** Tab index for focus */
    tabIndex?: number;
    /**
     * If is set and is different than Update.None forces component to recompute and call nodeGetter. This property
     * is not pure: component will re-render any time it receives this property and it is not Update.None.
     *
     * Useful if you changed something in the tree structure and want to re-render component using new data.
     */
    update?: Update;
    /** Width of Tree container */
    width: number;
}
export interface TreeState {
    order: string[];
    useDynamicRowHeight: boolean;
}
/**
 * This component renders a tree of elements using powerful virtualization technology to render
 * only visible elements to the HTML document.
 */
export default class Tree extends React.Component<TreeProps, TreeState> {
    static defaultProps: Partial<TreeProps>;
    state: TreeState;
    private grid;
    private registry;
    componentWillMount(): Promise<void>;
    componentWillReceiveProps({update}: TreeProps): Promise<void>;
    forceUpdateGrid(callback?: () => any): void;
    /** See Grid#getOffsetForCell */
    getOffsetForRow({alignment, index}: {
        alignment?: Alignment;
        index?: number;
    }): number;
    /** CellMeasurer compatibility */
    invalidateCellSizeAfterRender({columnIndex, rowIndex}: CellPosition): void;
    /** See Grid#measureAllCells */
    measureAllRows(): void;
    /** CellMeasurer compatibility */
    recomputeGridSize({columnIndex, rowIndex}?: Partial<CellPosition>): void;
    /**
     * Recomputes Tree component basing on update type.
     *
     * Method calls "nodeGetter" internally and flattens tree structure to an array. Depending on "update" value it runs
     * one of the following algorithms:
     * 1) Update.NodesAndOpenness. Requires full node metadata. Updates order, number and rendering data of all
     * nodes. Overrides current openness state with "openedByDefault" value.
     * 2) Update.Nodes. Requires full node metadata. Updates order, number and rendering data of all nodes. Preserves
     * openness state of all nodes.
     * 3) Update.Order. Requires only node id. Updates nodes order (useful for sorting etc.). Preserves openness
     * state of all nodes.
     * 4) Update.None. Updates nothing.
     *
     * @param update
     *
     * @returns promise resolves when re-rendering is ended
     */
    recomputeTree(update: Update): Promise<void>;
    render(): JSX.Element;
    /** See Grid#scrollToPosition */
    scrollToPosition(scrollTop?: number): void;
    /** See Grid#scrollToCell */
    scrollToRow(index?: number): void;
    shouldComponentUpdate({update, ...nextOther}: TreeProps, nextState: TreeState): boolean;
    /**
     * Overrides openness state of nodes width ids specified as map keys to map values respectively.
     * @param map
     */
    toggleNodes(map: {
        [id: string]: boolean;
    }): Promise<void>;
    private getRowHeight;
    private cellRenderer;
    private finishNodeToggling;
    private onScroll;
    private onSectionRendered;
    private setRef;
}
